.termynal-comment {
  color: #4a968f;
  font-style: italic;
  display: block;
}

.termy [data-termynal] {
  white-space: pre-wrap;
}

a.external-link::after {
  /* \00A0 is a non-breaking space
        to make the mark be on the same line as the link
    */
  content: "\00A0[↪]";
}

a.internal-link::after {
  /* \00A0 is a non-breaking space
        to make the mark be on the same line as the link
    */
  content: "\00A0↪";
}


div.autodoc-docstring {
  padding-left: 20px;
  margin-bottom: 30px;
  border-left: 5px solid rgba(230, 230, 230);
}

div.autodoc-members {
  padding-left: 20px;
  margin-bottom: 15px;
}
